//
//  main.c
//  Nonlinear Observer
//
//  Created by Fredrik Alvenes on 17.03.13.
//  Copyright (c) 2013 Fredrik Alvenes. All rights reserved.
//

#include <asf.h>
#include "sensor_demo.h"
#include <stdio.h>
#include <math.h>
#include <string.h>


//************************   PROTOTYPES   ************************

struct quaternion {
    float eta;
    float eps1;
    float eps2;
    float eps3;
};

struct vector {
    float x;
    float y;
    float z;
};

struct RK4_3D_storage_struct{       //Prototype struct for 3D Runge Kutta algorithm
    float fourth_Prev_1;
	float third_Prev_1;
	float second_Prev_1;
	float Prev_1;
    float fourth_Prev_2;
	float third_Prev_2;
	float second_Prev_2;
	float Prev_2;
    float fourth_Prev_3;
	float third_Prev_3;
	float second_Prev_3;
	float Prev_3;
};

struct RK4_quat_storage_struct{     //Prototype struct for quaternion Runge Kutta algorithm
    float fourth_Prev_1;			
	float third_Prev_1;
	float second_Prev_1;
	float Prev_1;					//eta
    float fourth_Prev_2;			
	float third_Prev_2;
	float second_Prev_2;
	float Prev_2;					//eps1
    float fourth_Prev_3;
	float third_Prev_3;
	float second_Prev_3;
	float Prev_3;					//eps2
    float fourth_Prev_4;
	float third_Prev_4;
	float second_Prev_4;
	float Prev_4;					//eps3
};

struct Euler_quat_storage_struct{
	float    eta;
	float    eps1;
	float    eps2;
	float	 eps3;
	float timestamp;
};

struct Euler_vector_storage_struct{
	float x;
	float y;
	float z;
	float timestamp;
};

struct quaternion_Rotation_matrix{
	float one_one;    float one_two;    float one_three;
	float two_one;    float two_two;    float two_three;
	float three_one;  float three_two;  float three_three;
};

struct euler_angles{
	float phi;
	float theta;
	float psi;
};

struct attitude_int{
	int32_t roll;
	int32_t pitch;
	int32_t yaw;
};

struct quaternion_int{
	int32_t eta;
	int32_t eps1;
	int32_t eps2;
	int32_t eps3;
};

//*****************************************************************



//**********************   GLOBAL VARIABLES  **********************

struct quaternion q_hat = {1, 0, 0, 0};

float k_acc = 1; //0.55;                                    //vex accelerometer estimate gain
float k_mag = 0.55;                                    //vex magnetometer  estimate gain
                
float Kp    = 1;                                          //observer proportional gain
float Ki    = 0.08;                                          //observer integral gain

struct RK4_3D_storage_struct Bi_stored = {  0,0,0,0,          //storage struct for Ki integrator
                                            0,0,0,0,
                                            0,0,0,0 };                                          

struct RK4_quat_storage_struct q_hat_stored = { 1,1,1,1,	//eta	 //storage struct for quaternion integrator
                                                0,0,0,0,	//eps1
                                                0,0,0,0,	//eps2
                                                0,0,0,0 };	//eps3

struct Euler_quat_storage_struct q_hat_stored_euler;
struct Euler_vector_storage_struct b_hat_stored_euler;
												
int lcd_count = 0;
int lcd_state = 1;

int timestep = 0;

float uptime = 0;

struct vector local_mag;
	
static sensor_t accelerometer;
static sensor_data_t acc_data = {.scaled = true};
//static sensor_data_t acc_data.scaled = true;
//sensor_set_range(&accelerometer, 2000 /* milli-g */);
//sensor_set_bandwidth(&accelerometer, 200 /* Hertz */);
	

static sensor_t gyroscope;
static sensor_data_t gyr_data = {.scaled = true};
//static sensor_data_t gyr_data.scaled = true;

sensor_t compass;
static sensor_data_t mag_data = {.scaled = true};
//static sensor_data_t mag_data.scaled = true;

//*****************************************************************



//***********************   RUNGE KUTTA 3D  ***********************
//4th order Runge Kutta integration algorithm 3-Dimension
struct vector RungeKutta_3D_integration(struct RK4_3D_storage_struct *rk, struct vector *a) {
    
    rk->Prev_1 = rk->Prev_1 +  0.16667 * ( rk->fourth_Prev_1 + 2*rk->third_Prev_1 + 2*rk->second_Prev_1 + a->x);
    rk->Prev_2 = rk->Prev_2 +  0.16667 * ( rk->fourth_Prev_2 + 2*rk->third_Prev_2 + 2*rk->second_Prev_2 + a->y);
    rk->Prev_3 = rk->Prev_3 +  0.16667 * ( rk->fourth_Prev_3 + 2*rk->third_Prev_3 + 2*rk->second_Prev_3 + a->z);
    
    rk->fourth_Prev_1 = rk->third_Prev_1;
	rk->third_Prev_1 = rk->second_Prev_1;
	rk->second_Prev_1 = a->x;
    
    rk->fourth_Prev_2 = rk->third_Prev_2;
	rk->third_Prev_2 = rk->second_Prev_2;
	rk->second_Prev_2 = a->y;
    
    rk->fourth_Prev_3 = rk->third_Prev_3;
	rk->third_Prev_3 = rk->second_Prev_3;
	rk->second_Prev_3 = a->z;
    
    struct vector integrated_3D = {rk->Prev_1,rk->Prev_2,rk->Prev_3};
    
    return integrated_3D;
}
//*****************************************************************



//*******************   RUNGE KUTTA quaternion  *******************
//4th order Runge Kutta integration algorithm for quaternion
struct quaternion RungeKutta_quat_integration(struct RK4_quat_storage_struct *rk, struct quaternion *q) {
    
    rk->Prev_1 = rk->Prev_1 +  0.16667 * ( rk->fourth_Prev_1 + 2*rk->third_Prev_1 + 2*rk->second_Prev_1 + q->eta);
    rk->Prev_2 = rk->Prev_2 +  0.16667 * ( rk->fourth_Prev_2 + 2*rk->third_Prev_2 + 2*rk->second_Prev_2 + q->eps1);
    rk->Prev_3 = rk->Prev_3 +  0.16667 * ( rk->fourth_Prev_3 + 2*rk->third_Prev_3 + 2*rk->second_Prev_3 + q->eps2);
    rk->Prev_4 = rk->Prev_3 +  0.16667 * ( rk->fourth_Prev_4 + 2*rk->third_Prev_4 + 2*rk->second_Prev_4 + q->eps3);
    
    rk->fourth_Prev_1 = rk->third_Prev_1;
	rk->third_Prev_1  = rk->second_Prev_1;
	rk->second_Prev_1 = q->eta;
    
    rk->fourth_Prev_2 = rk->third_Prev_2;
	rk->third_Prev_2  = rk->second_Prev_2;
	rk->second_Prev_2 = q->eps1;
    
    rk->fourth_Prev_3 = rk->third_Prev_3;
	rk->third_Prev_3  = rk->second_Prev_3;
	rk->second_Prev_3 = q->eps2;
    
    rk->fourth_Prev_4 = rk->third_Prev_4;
	rk->third_Prev_4  = rk->second_Prev_4;
	rk->second_Prev_4 = q->eps3;
    
    struct quaternion integrated_quat = {rk->Prev_1,rk->Prev_2,rk->Prev_3,rk->Prev_4};
    
    return integrated_quat;
}
//*****************************************************************


struct quaternion Forward_euler_quat_integration(struct quaternion *q_dot, int timestep) {
	
	float h = timestep;
	//h = h/31250; //1953.125;
	h = 0.01;
	
	q_hat_stored_euler.eta  = q_hat_stored_euler.eta  + h*q_dot->eta;
	q_hat_stored_euler.eps1 = q_hat_stored_euler.eps1 + h*q_dot->eps1;
	q_hat_stored_euler.eps2 = q_hat_stored_euler.eps2 + h*q_dot->eps2;
	q_hat_stored_euler.eps3 = q_hat_stored_euler.eps3 + h*q_dot->eps3;
	
	struct quaternion q_prev_return = {q_hat_stored_euler.eta, q_hat_stored_euler.eps1, q_hat_stored_euler.eps2, q_hat_stored_euler.eps3};
	
	return q_prev_return;
}

struct vector Forward_euler_vector_integration(struct vector *b_dot, int timestep) {
	
	float h = timestep;
	//h = h/31250; //1953.125;
	h = 0.01;
	
	b_hat_stored_euler.x = b_hat_stored_euler.x + h*b_dot->x;
	b_hat_stored_euler.y = b_hat_stored_euler.y + h*b_dot->y;
	b_hat_stored_euler.z = b_hat_stored_euler.z + h*b_dot->z;
	
	struct vector b_prev_return = {b_hat_stored_euler.x, b_hat_stored_euler.y, b_hat_stored_euler.z};
	
	return b_prev_return;
}

//STEP 1.1 Normalizes the accelerometer and magnetometer data
struct vector normalize_vec(struct vector *a){
    
    float a_length = sqrt(a->x * a->x + a->y * a->y + a->z * a->z);
    
    struct vector a_norm;
    
    a_norm.x = (a->x)/a_length;
    a_norm.y = (a->y)/a_length;
    a_norm.z = (a->z)/a_length;

    return a_norm;
}

//STEP 2: The vex algorithm (acc_normalized, mag_local, mag_normalized and q_hat as arguments)
struct vector vex(struct vector *a_n, struct vector *m_l, struct vector *m_n, struct quaternion *q){
    
    struct vector w_mes;             //return vector
    
    struct vector a_hat;             //estimated accelerometer measurement
    struct vector m_hat;             //estimated magnetometer measurement
    
    a_hat.x =     2 * ( q->eps1 * q->eps3  -  q->eps2 * q->eta  );
    a_hat.y =     2 * ( q->eps2 * q->eps3  +  q->eps1 * q->eta  );
    a_hat.z = 1 - 2 * ( q->eps1 * q->eps1  +  q->eps2 * q->eps2 );
    
    m_hat.x = (1 - 2 * (q->eps2 * q->eps2 + q->eps3 * q->eps3)) * m_l->x  +      (2 * (q->eps1 * q->eps2 + q->eps3 * q->eta))  * m_l->y  +      (2 * (q->eps1 * q->eps3 - q->eps2 * q->eta))   * m_l->z;
    m_hat.y =     (2 * (q->eps1 * q->eps2 - q->eps3 * q->eta))  * m_l->x  +  (1 - 2 * (q->eps1 * q->eps1 + q->eps3 * q->eps3)) * m_l->y  +      (2 * (q->eps2 * q->eps3 + q->eps1 * q->eta))   * m_l->z;
    m_hat.z =     (2 * (q->eps1 * q->eps3 + q->eps2 * q->eta))  * m_l->x  +      (2 * (q->eps2 * q->eps3 - q->eps1 * q->eta))  * m_l->y  +  (1 - 2 * (q->eps1 * q->eps1 + q->eps2 * q->eps2))  * m_l->z;
    
    //husk fortegn!!!
    
    
    w_mes.x = -k_acc * (a_n->z * a_hat.y - a_n->y * a_hat.z)  -  k_mag * (m_n->z * m_hat.y - m_n->y * m_hat.z);
    w_mes.y = -k_acc * (a_n->x * a_hat.z - a_n->z * a_hat.x)  -  k_mag * (m_n->x * m_hat.z - m_n->z * m_hat.x);
    w_mes.z = -k_acc * (a_n->y * a_hat.x - a_n->x * a_hat.y)  -  k_mag * (m_n->y * m_hat.x - m_n->x * m_hat.y);
    
    return w_mes;
}

//STEP 3: Multiply omega_mes with Ki gain
struct vector w_mes_Ki_gain(struct vector *w){
    
    struct vector Bi_dot;
    
    Bi_dot.x = w->x * (-Ki);
    Bi_dot.y = w->y * (-Ki);
    Bi_dot.z = w->z * (-Ki);
    
    return Bi_dot;
}

//STEP 5: Sum Gyroscope, Integral- and proportional-gain
struct vector sum_Gyr_Bi_Bp(struct vector *gyr, struct vector *Bi, struct vector *Bp){
    
    struct vector omega;
    
    omega.x = gyr->x - Bi->x + ( Bp->x ) * Kp;
    omega.y = gyr->y - Bi->y + ( Bp->y ) * Kp;
    omega.z = gyr->z - Bi->z + ( Bp->z ) * Kp;
    
    return omega;
}

//STEP 6: Generates q_hat_dot
struct quaternion q_hat_dot_generate(struct vector *w, struct quaternion *q){
    
    struct quaternion q_hat_dot;
    
    q_hat_dot.eta  = - 0.5 * (  (w->x) * (q->eps1) + (w->y) * (q->eps2) + (w->z) * (q->eps3));
    q_hat_dot.eps1 =   0.5 * (  (w->x) * (q->eta)  - (w->y) * (q->eps3) + (w->z) * (q->eps2));
    q_hat_dot.eps2 =   0.5 * (  (w->x) * (q->eps3) + (w->y) * (q->eta)  - (w->z) * (q->eps1));
    q_hat_dot.eps3 =   0.5 * ( -(w->x) * (q->eps2) + (w->y) * (q->eps1) + (w->z) * (q->eta) );
    
    return q_hat_dot;
}

//STEP 9 Normalizes the quaternion
struct quaternion normalize_q(struct quaternion *q){
    
    float q_length = sqrt(q->eta * q->eta + q->eps1 * q->eps1 + q->eps2 *  q->eps2 + q->eps3 *  q->eps3);
    
    struct quaternion q_norm;
    
    q_norm.eta =  (q->eta )/q_length;
    q_norm.eps1 = (q->eps1)/q_length;
    q_norm.eps2 = (q->eps2)/q_length;
    q_norm.eps3 = (q->eps3)/q_length;
    
    return q_norm;
}

//STEP 10 quaternion to euler angles in degrees
struct euler_angles q2euler(struct quaternion *q){
	
	struct quaternion_Rotation_matrix R;
	
	R.one_one = (1 - 2 * (q->eps2 * q->eps2 + q->eps3 * q->eps3));      R.one_two =     (2 * (q->eps1 * q->eps2 - q->eps3 * q->eta));        R.one_three =       (2 * (q->eps1 * q->eps3 + q->eps2 * q->eta));
	R.two_one =     (2 * (q->eps1 * q->eps2 + q->eps3 * q->eta));       R.two_two = (1 - 2 * (q->eps1 * q->eps1 + q->eps3 * q->eps3));       R.two_three =       (2 * (q->eps2 * q->eps3 - q->eps1 * q->eta));
	R.three_one =   (2 * (q->eps1 * q->eps3 - q->eps2 * q->eta));       R.three_two =   (2 * (q->eps2 * q->eps3 + q->eps1 * q->eta));        R.three_three = (1 - 2 * (q->eps1 * q->eps1 + q->eps2 * q->eps2));
	
	float phi   = atan2f (R.three_two,R.three_three) * 180 / PI;
	float theta = -asinf(R.three_one) * 180 / PI;
	float psi   = atan2f(R.two_one,R.one_one)  * 180 / PI;
	
struct euler_angles attitude = {phi, theta, psi};

return attitude;
}


enum format_string_select {
	attitude_format	    = 0,
};



static void draw_formatted_vector(enum format_string_select sel, struct attitude_int *data){

	static char format_buffer[3][17];

	static const char *format[] = {

		/* Attitude */
		"%5ld",
		"%5ld",
		"%5ld"
	};

	LED_Toggle(LED0);
	LED_Toggle(LED1);


	sprintf(format_buffer[0], format[sel + 0], data->roll);
	sprintf(format_buffer[1], format[sel + 1], data->pitch);
	sprintf(format_buffer[2], format[sel + 2], data->yaw);

	gfx_mono_draw_string(format_buffer[0], 1, 5, &sysfont);
	gfx_mono_draw_string(format_buffer[1], 1, 13, &sysfont);
	gfx_mono_draw_string(format_buffer[2], 1, 21, &sysfont);

	//delay_ms(hz_to_ms(REFRESH_PERIOD));
}

static void draw_formatted_quaternion(enum format_string_select sel, struct quaternion_int *data){

	static char format_buffer[4][17];

	static const char *format[] = {

		/* quaternion */
		"%5ld",
		"%5ld",
		"%5ld",
		"%5ld"
	};
	
	LED_Toggle(LED0);
	LED_Toggle(LED1);
	
	sprintf(format_buffer[0], format[sel + 0], data->eta);
	sprintf(format_buffer[1], format[sel + 1], data->eps1);
	sprintf(format_buffer[2], format[sel + 2], data->eps2);
	sprintf(format_buffer[3], format[sel + 3], data->eps3);

	gfx_mono_draw_string(format_buffer[0], 0, 0, &sysfont);
	gfx_mono_draw_string(format_buffer[1], 0, 8, &sysfont);
	gfx_mono_draw_string(format_buffer[2], 0, 16, &sysfont);
	gfx_mono_draw_string(format_buffer[3], 0, 24, &sysfont);

	//delay_ms(hz_to_ms(REFRESH_PERIOD));
}

static void loading_screen(){
	clear_screen();
	
	gfx_mono_draw_rect(1, 1, 126, 30,GFX_PIXEL_SET);

	for(int i = 1; i < 150; i=i+31)
	gfx_mono_draw_filled_rect(1, 1, i, 30, GFX_PIXEL_SET);
	delay_ms(20);
}

static void euler_screen(){
		clear_screen();
		screen_border();
		
		gfx_mono_draw_string("         Roll  (deg)", 1, 5, &sysfont);
		gfx_mono_draw_string("         Pitch (deg)", 1, 13, &sysfont);
		gfx_mono_draw_string("         Yaw   (deg)", 1, 21, &sysfont);
}

static void bias_screen(){
		clear_screen();
		screen_border();
		
		gfx_mono_draw_string("       X Bias (deg/s)", 1, 5, &sysfont);
		gfx_mono_draw_string("       Y Bias (deg/s)", 1, 13, &sysfont);
		gfx_mono_draw_string("       Z Bias (deg/s)", 1, 21, &sysfont);	
}

static void sigma_screen(){
	clear_screen();
	screen_border();
	
	gfx_mono_draw_string("       X Sigma (deg)", 1, 5, &sysfont);
	gfx_mono_draw_string("       Y Sigma (deg)", 1, 13, &sysfont);
	gfx_mono_draw_string("       Z Sigma (deg)", 1, 21, &sysfont);
}

static void quaternion_screen(){
		clear_screen();
		
		gfx_mono_draw_string("        eta  (X 1000)", 1, 0, &sysfont);
		gfx_mono_draw_string("        eps1 (X 1000)", 1, 8, &sysfont);
		gfx_mono_draw_string("        eps2 (X 1000)", 1, 16, &sysfont);
		gfx_mono_draw_string("        eps3 (X 1000)", 1, 24, &sysfont);
}

static void accelerometer_screen(){
	clear_screen();
	screen_border();
	
	gfx_mono_draw_string("          X acc (mg)", 1, 5, &sysfont);
	gfx_mono_draw_string("          Y acc (mg)", 1, 13, &sysfont);
	gfx_mono_draw_string("          Z acc (mg)", 1, 21, &sysfont);
}

static void magnetometer_screen(){
	clear_screen();
	screen_border();
	
	gfx_mono_draw_string("          X mag (uT)", 1, 5, &sysfont);
	gfx_mono_draw_string("          Y mag (uT)", 1, 13, &sysfont);
	gfx_mono_draw_string("          Z mag (uT)", 1, 21, &sysfont);
}

static void gyroscope_screen(){
	clear_screen();
	screen_border();
	
	gfx_mono_draw_string("      X gyr (deg/sec)", 1, 5, &sysfont);
	gfx_mono_draw_string("      Y gyr (deg/sec)", 1, 13, &sysfont);
	gfx_mono_draw_string("      Z gyr (deg/sec)", 1, 21, &sysfont);
}

static void frequency_screen(){
	clear_screen();
	screen_border();
	gfx_mono_draw_string(" Frequency", 1, 5, &sysfont);
	gfx_mono_draw_string("Uptime", 76, 5, &sysfont);
	//gfx_mono_draw_string("      Y gyr (deg/sec)", 1, 13, &sysfont);
	gfx_mono_draw_string("      Hz", 1, 21, &sysfont);
	gfx_mono_draw_string("      s", 80, 21, &sysfont);
	gfx_mono_draw_rect(63,1, 1, 30, GFX_PIXEL_SET);
}

static struct vector gyroscope_readout(){
		gyr_data.scaled = true;
		sensor_read(&gyroscope, SENSOR_READ_ROTATION, &gyr_data);
		
		int x_gyr = gyr_data.axis.x;
		int y_gyr = gyr_data.axis.y;
		int z_gyr = gyr_data.axis.z;
		
		float x_gyr_f = x_gyr;
		float y_gyr_f = y_gyr;
		float z_gyr_f = z_gyr;
		 
		struct vector gyro = {x_gyr_f*PI/180, y_gyr_f*PI/180, z_gyr_f*PI/180};
			
	return gyro; 
}

static struct vector accelerometer_readout(){
		
		sensor_read(&accelerometer, SENSOR_READ_ACCELERATION, &acc_data);
		
		int x_acc = acc_data.axis.x;
		int y_acc = acc_data.axis.y;
		int z_acc = acc_data.axis.z;
		
		float x_acc_f = x_acc;
		float y_acc_f = y_acc;
		float z_acc_f = z_acc;
		
		struct vector acc = {x_acc_f/1000, y_acc_f/1000, z_acc_f/1000};
	
	return acc;
}

static struct vector compass_readout(){
		sensor_set_bandwidth(&compass, 75 /* Hertz */);
		sensor_read(&compass, SENSOR_READ_FIELD, &mag_data);
		
		int x_mag = mag_data.axis.x;
		int y_mag = mag_data.axis.y;
		int z_mag = mag_data.axis.z;
		
		float x_mag_f = x_mag;
		float y_mag_f = y_mag;
		float z_mag_f = z_mag;
		
		struct vector mag = {x_mag_f, y_mag_f, z_mag_f};
	
	return mag;
}

static void screen_output(struct quaternion *q, struct vector *bias, struct vector *gyr, struct vector *acc, struct vector *mag, struct vector *sigma, int timestep, float uptime){
		if ((fabs(bias->x - gyr->x) + fabs(bias->y - gyr->y) + fabs(bias->z - gyr->z)) < 0.03){
			gpio_set_pin_high(LED2);
			gpio_set_pin_low(LED3);
		}else{
			gpio_set_pin_low(LED2);
			gpio_set_pin_high(LED3);
		}
		if ( lcd_state == 1){
			//Transform quaternion to Euler angles
			struct euler_angles attitude = q2euler(&q_hat);
			
			//Cast ROLL, PITCH and YAW to integers for print on screen
			int32_t roll_int = (int32_t) attitude.phi;
			int32_t pitch_int = (int32_t) attitude.theta;
			int32_t yaw_int = (int32_t) attitude.psi;
			
			struct attitude_int attitudeINT = {roll_int, pitch_int, yaw_int};
		
			draw_formatted_vector(attitude_format, &attitudeINT);
		
			lcd_count = 0;
		}
		if ( lcd_state == 2 ){
				bias->x = bias->x*180/PI;
				bias->y = bias->y*180/PI;
				bias->z = bias->z*180/PI;
				
				int32_t bias_x_int = (int32_t) bias->x;
				int32_t bias_y_int = (int32_t) bias->y;
				int32_t bias_z_int = (int32_t) bias->z;
				
			struct attitude_int biasINT = {bias_x_int, bias_y_int, bias_z_int};
			
			draw_formatted_vector(attitude_format, &biasINT);
			
			lcd_count = 0;
		}	
		if ( lcd_state == 3 ){
			sigma->x = sigma->x*Ki*180/PI;
			sigma->y = sigma->y*Ki*180/PI;
			sigma->z = sigma->z*Ki*180/PI;
			
			int32_t sigma_x_int = (int32_t) sigma->x;
			int32_t sigma_y_int = (int32_t) sigma->y;
			int32_t sigma_z_int = (int32_t) sigma->z;
			
		struct attitude_int sigmaINT = {sigma_x_int, sigma_y_int, sigma_z_int};
		
		draw_formatted_vector(attitude_format, &sigmaINT);
		
		lcd_count = 0;
		}
		if ( lcd_state == 4 ) {
			struct quaternion qINT = {q_hat.eta*1000, q_hat.eps1*1000, q_hat.eps2*1000, q_hat.eps3*1000};
		
			int32_t eta_int  = (int32_t) qINT.eta;
			int32_t eps1_int = (int32_t) qINT.eps1;
			int32_t eps2_int = (int32_t) qINT.eps2;
			int32_t eps3_int = (int32_t) qINT.eps3;
		
			struct quaternion_int quaternionINT = {eta_int, eps1_int, eps2_int, eps3_int};
	
			draw_formatted_quaternion(attitude_format, &quaternionINT);
	
			lcd_count = 0;
		}
		if ( lcd_state == 5 ) {
			//Cast ROLL, PITCH and YAW to integers for print on screen
			gyr->x = gyr->x*180/PI;
			gyr->y = gyr->y*180/PI;
			gyr->z = gyr->z*180/PI;
			
			int32_t gyr_x_int = (int32_t) gyr->x;
			int32_t gyr_y_int = (int32_t) gyr->y;
			int32_t gyr_z_int = (int32_t) gyr->z;
			
			struct attitude_int gyrINT = {gyr_x_int, gyr_y_int, gyr_z_int};
		
			draw_formatted_vector(attitude_format, &gyrINT);
			
			lcd_count = 0;
		}
		if ( lcd_state == 6 ) {
			//Cast ROLL, PITCH and YAW to integers for print on screen
			acc->x = acc->x*1000;
			acc->y = acc->y*1000;
			acc->z = acc->z*1000;
			
			int32_t acc_x_int = (int32_t) acc->x;
			int32_t acc_y_int = (int32_t) acc->y;
			int32_t acc_z_int = (int32_t) acc->z;
			
			struct attitude_int accINT = {acc_x_int, acc_y_int, acc_z_int};
		
			draw_formatted_vector(attitude_format, &accINT);
			
			lcd_count = 0;
		}
		if ( lcd_state == 7 ) {
			int32_t mag_x_int = (int32_t) mag->x;
			int32_t mag_y_int = (int32_t) mag->y;
			int32_t mag_z_int = (int32_t) mag->z;
			
			struct attitude_int magINT = {mag_x_int, mag_y_int, mag_z_int};
		
			draw_formatted_vector(attitude_format, &magINT);
			
			lcd_count = 0;
		}
		if ( lcd_state == 8 ) {
			float h = timestep;
			h = h/31250; //1953.125;
			h = 1/h;
			
			float uptime_temp = uptime/31250;
			
			int32_t uptime_int = (int32_t) uptime_temp;
			static char *uptime_char;
			sprintf(uptime_char, "%5ld", uptime_int);
			gfx_mono_draw_string(uptime_char, 80, 21, &sysfont);
			
			int32_t freq = (int32_t) h;
			static char *freq_char;
			sprintf(freq_char, "%5ld", freq);
			gfx_mono_draw_string(freq_char, 1, 21, &sysfont);
			
			LED_Toggle(LED0);
			LED_Toggle(LED1);
			lcd_count = 0;
		}		
}

static void screen_state_update(){
		lcd_state++;
		if (lcd_state > 8)
		{
			lcd_state = 1;
		}
		
		loading_screen();
		switch ( lcd_state ) {
			case 1:
				euler_screen();
				break;
			case 2:
				bias_screen();
				break;
			case 3:
				sigma_screen();
				break;
			case 4:
				quaternion_screen();
				break;
			case 5:
				gyroscope_screen();
				break;
			case 6:
				accelerometer_screen();
				break;
			case 7:
				magnetometer_screen();
				break;
			case 8:
				frequency_screen();
				break;
		}			
}

static void prompt_user(char *prompt_string){
    
	gfx_mono_draw_string(prompt_string, 1, 5, &sysfont);
	gfx_mono_draw_string("and press SW1", 1, 13, &sysfont);
    
    
	/* Wait for user to push button before continuing. */
	do {
		delay_ms(500);
	} while (!switch_pressed(SW1));
    
    
	/* Wait for button release before continuing. */
	while (switch_pressed(SW1)) {
	}
    
	clear_screen();
}

//System startup with initializations loading screen etc.
static void boot_up(){
    
    //Setting up the system with prescaler
    pmic_init();
    sysclk_init();
    sensor_platform_init();
    tc_enable(&TCC0);
    tc_write_period(&TCC0, 65000); //Number of counts before reset
    tc_write_clock_source(&TCC0, TC_CLKSEL_DIV1024_gc); //Pre-scaler 32MHz / 1024
    
    
    sensor_attach(&accelerometer, SENSOR_TYPE_ACCELEROMETER, 0, 0);
    sensor_attach(&gyroscope, SENSOR_TYPE_GYROSCOPE, 0, 0);
    sensor_attach(&compass, SENSOR_TYPE_COMPASS, 0, 0);
    gfx_mono_init();
    
    //gpio_set_pin_high(NHD_C12832A1Z_BACKLIGHT);
    st7565r_set_contrast(ST7565R_DISPLAY_CONTRAST_MIN);
    
    gfx_mono_draw_string("  Nonlinear Observer", 1, 13, &sysfont);
    
    delay_ms(1000);
    
    clear_screen();
    
    gfx_mono_draw_string("   www.alvenes.com", 1, 13, &sysfont);
    
    delay_ms(500);
    loading_screen();
    LED_Toggle(LED0);
}

static void calibrate_compass(){
	LED_Toggle(LED2);
	LED_Toggle(LED3);
	clear_screen();
    /* Calibration Step 1. */
	prompt_user("Lay IMU flat");
	(void)sensor_calibrate(&compass, MANUAL_CALIBRATE, 1, NULL);
    
	/* Calibration Step 2. */
	prompt_user("Rotate flat 180");
	(void)sensor_calibrate(&compass, MANUAL_CALIBRATE, 2, NULL);
    
	/* Calibration Step 3. */
	prompt_user("Flip IMU upside down");
    
	if (sensor_calibrate(&compass, MANUAL_CALIBRATE, 3, NULL) != true) {
		if (compass.err == SENSOR_ERR_IO) {
			gfx_mono_draw_string("Calibration failure", 1, 5,
                                 &sysfont);
			gfx_mono_draw_string("Write Error", 1, 13, &sysfont);
		} else {
			gfx_mono_draw_string("Unknown Error", 1, 5, &sysfont);
			gfx_mono_draw_string("Calibrating Device", 1, 13,
                                 &sysfont);
		}
        
	}
    
	LED_Toggle(LED3);
}

int main(void)
{
		//Initalize board and sensors and everything else
		board_init();
		boot_up();
		
		//NED magnetic reference for magnetometer
		calibrate_compass();
		local_mag.y = 13.5961;		//Algning Y with North
		local_mag.x = 0.5467;		//Aligning X with East
		local_mag.z = -49.9338;		//Aligning Z with negative Down
			
		struct vector local_mag_norm = normalize_vec(&local_mag);
		
		//Initializing integrator
		q_hat_stored_euler.eta = 1;
		q_hat_stored_euler.eps1 = 1;
		q_hat_stored_euler.eps2 = 1;
		q_hat_stored_euler.eps3 = 1;
		q_hat_stored_euler.timestamp = 0;
		
		b_hat_stored_euler.x = 0;
		b_hat_stored_euler.y = 0;
		b_hat_stored_euler.z = 0;
		b_hat_stored_euler.timestamp = 0;
		
		gpio_configure_pin(J4_PIN0, IOPORT_DIR_OUTPUT);
		
		//Set up the first screen paramters
		euler_screen();
	
	
	while (true) {
		
		//Read one loop time from stopwatch, then reset stopwatch
        timestep = tc_read_count(&TCC0);
		uptime = uptime + timestep;
		tc_write_count(&TCC0, 0);
		
		/*************	STEP 1 *************/
		//Read and store gyroscope sensor data
		struct vector gyr = gyroscope_readout();
		
		//Read and store and accelerometer sensor data
		struct vector acc = accelerometer_readout();
		
		//Read and store  magnetometer sensor data
		struct vector mag = compass_readout();
		
		//Normalize sensor data
		struct vector acc_norm  = normalize_vec(&acc);
		struct vector mag_norm  = normalize_vec(&mag);
		
		/*************	STEP 2 *************/
		//Vex algorithm
		struct vector omega_mes = vex(&acc_norm, &local_mag_norm, &mag_norm, &q_hat);
		
		/*************	STEP 3 *************/
		//Gaining w_mes output with Ki to create Bi_dot
		struct vector bias_dot = w_mes_Ki_gain(&omega_mes);
		
		/*************	STEP 4 *************/
		//Numerically integrate Bi_dot
		//struct vector bias = RungeKutta_3D_integration(&Bi_stored, &bias_dot);
		struct vector bias = Forward_euler_vector_integration(&bias_dot, timestep);
		
		/*************	STEP 5 *************/
		//Creating the sum of Gyroscope, Bi and Omega_mes
		struct vector injection = sum_Gyr_Bi_Bp(&gyr, &bias, &omega_mes);
		
		/*************	STEP 6 *************/
		//Creating the q_hat_dot with the T(q) matrix
		struct quaternion q_hat_dot = q_hat_dot_generate(&injection, &q_hat);
		
		/*************	STEP 7 *************/
		//Numerically integrate q_hat_dot with feedback from q_hat
		q_hat = Forward_euler_quat_integration(&q_hat_dot, timestep);
		//q_hat = RungeKutta_quat_integration(&q_hat_stored, &q_hat_dot);
		
		/*************	STEP 8 *************/
		//Normalizing the quaternion
		q_hat = normalize_q(&q_hat);
		
		//Output attitude to screen every given interval
		if (lcd_count == 10){
			screen_output(&q_hat, &bias, &gyr, &acc, &mag, &omega_mes, timestep, uptime);
		}else{
			lcd_count++;
		}		
		
		//Update screen state if  button-1 is pressed
		if (switch_pressed(SW1)){
			screen_state_update();
		}
		
		if (switch_pressed(SW2)){
			gpio_toggle_pin(NHD_C12832A1Z_BACKLIGHT);
			delay_ms(500);
		}
		
		//Benchmark readout pin toggle
		gpio_toggle_pin(J4_PIN0);
	
	}
}	



