clc
clear all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters for earth and orbit %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m  = 2.6;                   % [kg] Satellite mass
dx = 0.11;                  % X-axis length
dy = 0.1;                   % Y-axis length
dz = 0.2;                   % Z-axis length
Re = 6371.2e3;              % [m] Earth radius
Rs = 600e3;                 % [m] Satellite altitude
Rc = Re+Rs;                 % [m] Distance from earth center to satellite
G = 6.67428e-11;            % Earth gravitational constant
M = 5.972e24;               % Earth mass
w_O = sqrt(G*M/Rc^3);       % Satellite angular velocity relative Earth
w_O_IO = [0 -w_O 0]';       % Velocity of orbit coordinate system relative ECI


%%%%%%%%%%%%%%%%%%%%%%
% Moments of inertia %
%%%%%%%%%%%%%%%%%%%%%%

Ix = (m/12)*(dy^2+dz^2);    % X-axis inertia
Iy = (m/12)*(dx^2+dz^2);    % Y-axis inertia
Iz = (m/12)*(dx^2+dy^2);    % Z-axis inertia

% The inertia matrix
Inertia = diag([Ix Iy Iz]);


%%%%%%%%%%%%%%%%%%%%%%%
% Initial orientation %
%%%%%%%%%%%%%%%%%%%%%%%

% Set the satellites inital attitude in degrees (set by user)
roll_degrees_0      = 10;
pitch_degrees_0     = 5;
yaw_degrees_0       = -2;

%roll_degrees_0      = 0;
%pitch_degrees_0     = 0;
%yaw_degrees_0       = 0;

% Converting to radians
roll = (pi/180)*roll_degrees_0;
pitch = (pi/180)*pitch_degrees_0;
yaw = (pi/180)*yaw_degrees_0;

% Converting to quaternions (set as inital value in Kinematics integrator)
q_0 = euler2q(roll, pitch, yaw);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial angular velocity %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Initial angular velocity in radians per second (set by user)
%w_B_OB_0 = [0 0 0]';
%w_B_OB_0 = [0.0005 0.0003 -0.003]';
w_B_OB_0 = [0.1 -0.2 0.05]';

%Calculates inital rotation matrix from body to orbit
%(This rotation matrix = I if q_0 = [+-1 0 0 0])
R_o_b = Rquat(q_0);
R_b_o = R_o_b';

%c1 is the first column vector in R_b_o
%This can be used to calculate the inital 
%angular velocity relative intertia frame
c2 = R_b_o(:,2);
c1 = R_b_o(:,1);
w_B_IB_0 = w_B_OB_0+w_O*c1


%%%%%%%%%%%%%%%%%%%
% Regulator gains %
%%%%%%%%%%%%%%%%%%%

%K_p = 0;
%K_d = 4e-5;

K_p = 5e-8;
K_d = 4e-5;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coil parameters and controll allocation matrix %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Number of turns with copper thread in each coil
Nx = 355;
Ny = 355;
Nz = 800;

% Area of each coil
Ax = 0.08*0.18;
Ay = 0.08*0.18;
Az = 0.08^2;

% Resistance in each coil
Rx = 110;
Ry = 110;
Rz = 110;

% K matrix - controll allocation matrix for regulator
K = [(Nx*Ax)/Rx 0 0; 0 (Ny*Ay)/Ry 0; 0 0 (Nz*Az)/Rz];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The dipole megnetic field of Earth %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Permeability of vacuum
my = 1e17/(4*pi);

% Magnitude of magnetic field
B_null = my/(Rc^3);












