%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plots of the data collected from the Simulink simulator %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set(0,'DefaultAxesFontSize',22)

% The time-vector collected from an arbritary signal
% t = w.Time;

%Scale the signal to get orbits on x-axis
orbitPeriod = (2*pi)/(w_O);     	
scalePlot = 1/orbitPeriod;       	
t = w.Time*scalePlot;

% Scale time to get latitude on x-axis
%t = w.Time*w_O;

%%%%%%%%%%%%%%%%%%%%
% Angular velocity %
%%%%%%%%%%%%%%%%%%%%


figure(1); clf
plot(t, w.Data(1,:), t, w.Data(2,:), t, w.Data(3,:))
xlabel('orbits')
ylabel('degrees/sec')
title('Angular veolcity \omega')
legend('roll','pitch','yaw')
grid ON

%%%%%%%%%%%%
% Attitude %
%%%%%%%%%%%%

figure(2); clf
plot(t, phi.Data, t, theta.Data, t, psi.Data)
xlabel('orbits')
ylabel('degrees')
title('Attitude in degrees')
legend('roll','pitch','yaw')
grid ON

%%%%%%%%%%%%%%%%%%%%%%
% Estimated attitude %
%%%%%%%%%%%%%%%%%%%%%%

figure(3); clf
plot(t, phihat.Data, t, thetahat.Data, t, psihat.Data)
xlabel('orbits')
ylabel('degrees')
title('Estimated attitude in degrees')
legend('roll','pitch','yaw')
grid ON

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimated attitude versus real attitude %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(4); clf
subplot(3,1,1), plot(t, phiVS.Data)
legend('roll')
title('Estimated versus real attitude in degrees')

subplot(3,1,2), plot(t, thetaVS.Data)
legend('pitch')
ylabel('Degrees')

subplot(3,1,3), plot(t, psiVS.Data)
legend('yaw')

xlabel('Orbits')
grid ON

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gyroscope bias estimate %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(5); clf
p = plot(t, gyro_bias.Data(1,:), t, gyro_bias.Data(2,:), t, gyro_bias.Data(3,:))
xlabel('orbits')
ylabel('degrees/sec')
title('Gyro bias estimate')
legend('roll','pitch','yaw')
set(p,'LineWidth',2)
grid ON

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Magnetic field in body %
%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(6); clf
plot(t, B_b.Data(1,:), t, B_b.Data(2,:), t, B_b.Data(3,:))
xlabel('orbits')
ylabel('Tesla (T)')
title('Magnetic flux field in BODY')
legend('x','y','z')
grid ON

%%%%%%%%%%
% Torque %
%%%%%%%%%%

figure(7); clf
plot(t, tau_m.Data(1,:), t, tau_m.Data(2,:), t, tau_m.Data(3,:))
xlabel('orbits')
ylabel('Newton Meter')
title('Torque')
legend('x','y','z')
grid ON


%%%%%%%%%%%%%%%
% Quaternions %
%%%%%%%%%%%%%%%

figure(8); clf
subplot(4,1,1), plot(t, q.Data(1,:))
legend('\eta')
title('Quaternions')

subplot(4,1,2), plot(t, q.Data(2,:))
legend('\epsilon 1')

subplot(4,1,3), plot(t, q.Data(3,:))
legend('\epsilon 2')

subplot(4,1,4), plot(t, q.Data(4,:))
legend('\epsilon 3')
xlabel('orbits')
grid ON


%%%%%%%%%%%
% Voltage %
%%%%%%%%%%%

figure(9); clf
plot(t, V.Data(1,:), t, V.Data(2,:), t, V.Data(3,:))
xlabel('orbits')
ylabel('Volts')
title('Volts set up by actuators')
legend('x','y','z')
grid ON


%%%%%%%%%%%%
% Batttery %
%%%%%%%%%%%%

figure(10); clf
p = plot(t, Battery.Data)
xlabel('orbits')
ylabel('Percent')
title('Battery level')
set(p,'LineWidth',2)
grid ON


%%%%%%%%
% Watt %
%%%%%%%%

figure(11); clf
plot(t, Watt.Data)
xlabel('orbits')
ylabel('Watt')
title('Electrical energy use')
grid ON


